/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ddr3IopAsm.S
 *          Title:  DDR3 IOP assembly functions
 *     Programmer:  Nilesh Wilanker
 *     Modified  :  Naresh Madhusudana
 *  Creation Date:  July 4, 2011
 *
 *  Version History
 *  ---------------
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  07/04/11  NW    Initial version.
 *  10/06/11  NM    Porting this source to Invader
 *
 * Description
 * ------------
 * This file contains the assembly implementation of IOP functions needed by
 * DDR3 library.
 *
 *-------------------------------------------------------------------------
 */

#include <invader.h>

/* ***************************************************************************
 *
 *      I O P - F U N C T I O N S - S T A R T
 *
 * **************************************************************************/

#define PPC476_TCR_DIE              (0x04000000)   /* Enable decrementer interrupt */
#define PPC476_MSR_CE               (0x00020000)   /* Critical interrupt enable */
#define PPC476_MSR_EE               (0x00008000)   /* External interrupt enable */
#define PPC476_MSR_ME               (0x00001000)   /* Machine check enable */
#define IOC_PPC476_INTERRUPTS       (PPC476_MSR_ME |PPC476_MSR_EE | PPC476_MSR_CE)

#define PPC476_CACHE_LINE_SIZE      (128)         /* 32 bytes */
#define PPC476_CACHE_LINE_SIZE_MASK (0xFFFFFF80)   /* Mask is equal to -(PPC476_CACHE_LINE_SIZE) */

/* Stack pointer (sp) is r1 register in case of PPC:
 *     refer to http://www.ibm.com/developerworks/linux/library/l-powarch/
 *     search for stack
 *     Or refer to: http://www.csd.uwo.ca/~mburrel/stuff/ppc-asm.html
 */
#define sp r1

#define lwi2(reg, value)           \
        lis     reg, (value)@h;

#define lwi(reg, value)            \
        lis     reg, (value)@h;    \
        ori     reg, reg, (value)@l;


/*
 * iopinitFlushCache(AddressBase, Bytes)
 * Description:
 *     This function will flush all of the cache lines associated with
 *     given address range.  Start address need not be aligned to a CL boundary.
 *     Bytes need not be a CL multiple.
 * Arguments
 *     r3 - AddressBase
 *     r4 - Bytes
 * Other used registers
 *     r5 - CLS address mask
 *     r6 - Aligned end address
 */
        .globl iopinitFlushCache
iopinitFlushCache:
        subi    sp,sp,8
        stw     r5,0x0(sp)                      /* Save off r5 */
        stw     r6,0x4(sp)                      /* Save off r6 */
        lwi     (r5,PPC476_CACHE_LINE_SIZE_MASK)/* Load Cache Line Address Mask */
        add     r4,r4,r3                        /* Calculate unaligned end address */
        and     r3,r3,r5                        /* Align AddressBase */
        and     r6,r4,r5                        /* Calculate Aligned end address */
        cmp     0,r6,r4
        bnl     iopEndAddress                   /* If aligned address is less than the end address: */
        addi    r6,r6,PPC476_CACHE_LINE_SIZE    /* Then flush an additional cache line */
iopEndAddress:
        msync                                   /* Ensure that previous processor data activity completes */
iopFlushNextLine2:
        dcbst   0,r3                            /* Flush the next dcache line from L1 and L2*/
	msync
        dcbst   0,r3                            /* Flush the next dcache line from L3 (from L3 Ref manual)*/
        addi    r3,r3,PPC476_CACHE_LINE_SIZE    /* Go to next cache line */
        cmp     0,r3,r6                         /* If this cache line is less than aligned, end address: */
        blt     iopFlushNextLine2               /* Dump another cache line */
        lwz     r5,0x0(sp)                      /* Otherwise, restore r5,r6 and sp */
        lwz     r6,0x4(sp)
        addi    sp,sp,8
        msync                                   /* ensure that previous processor data activity completes*/
        blr

/*
 * iopinitCacheInvalidate(AddressBase, Bytes)
 * Description:
 *     This function will invalidate all of the cache lines associated with
 *     given address range.  Start address need not be aligned to a CL boundary.
 *     Bytes need not be a CL multiple.
 * Arguments
 *     r3 - AddressBase
 *     r4 - Bytes
 *     r5 - CLS address mask
 *     r6 - Aligned end address
 */
        .globl iopinitCacheInvalidate
iopinitCacheInvalidate:
        subi    sp,sp,8
        stw     r5,0x0(sp)                      /* Save off r5 */
        stw     r6,0x4(sp)                      /* Save off r6 */
        lwi     (r5,PPC476_CACHE_LINE_SIZE_MASK)/* Load Cache Line Address Mask */
        add     r4,r4,r3                        /* Calculate unaligned end address */
        and     r3,r3,r5                        /* Align AddressBase */
        and     r6,r4,r5                        /* Calculate Aligned end address */
        cmp     0,r6,r4
        bnl     iopInvEndAddress                /* If aligned address is less than the end address: */
        addi    r6,r6,PPC476_CACHE_LINE_SIZE    /* Then flush an additional cache line */
iopInvEndAddress:
        msync                                   /* Ensure that previous processor data activity completes */
iopInvNextLine2:
        dcbi    0,r3                            /* Invalidate the next dcache line from L1 and L2*/
	msync
        dcbi    0,r3                            /* Invalidate the next dcache line from L3 (from L3 ref manual)*/
        addi    r3,r3,PPC476_CACHE_LINE_SIZE    /* Go to next cache line */
        cmp     0,r3,r6                         /* If this cache line is less than aligned, end address: */
        blt     iopInvNextLine2                 /* Dump another cache line */
        lwz     r5,0x0(sp)                      /* Otherwise, restore r5,r6 and sp */
        lwz     r6,0x4(sp)
        addi    sp,sp,8
        msync                                   /* ensure that previous processor data activity completes */
        blr

/*
 * U32 iopinitDisableIRQ(void)
 * This subroutine may be called from C or assembly to disable the
 * PPC Decrmenter, Critical and External interrupts
 *
 * NOTE: R3-R6 are all volatile so it is OK for this routine to destroy them
 *
 * INPUT: void
 * OUTPUT:
 * Irq's that were cleared by this routine
 */
        .global iopinitDisableIRQ
iopinitDisableIRQ:
        /* Disable all PPC regular interrupts */
        mfmsr   r3
        mr      r5,r3
        mr      r6,r3
        lwi     (r4, IOC_PPC476_INTERRUPTS)
        and     r6,r6,r4                        /* keep current state to return */
        andc    r5, r5, r4
        mtmsr   r5                              /* Turn off 476 irqs */
        /* Also disable PPC decrementer interrupt */
        mftcr   r3
        lwi2    (r4,(PPC476_TCR_DIE))
        and     r5,r3,r4                        /* Retrieve DIE state */
        or      r5,r6,r5                        /* Or results */
        andc    r3,r3,r4
        mttcr   r3                              /* turn off DIE irq */
        mr      r3,r5                           /* Set up return register r3 */
        blr

/*
 * void iopinitEnableIRQ(U32 IrqMask)
 * This subroutine may be called from C or assembly to re-enable the
 * PPC Decrmenter, Critical and External interrupts
 *
 * NOTE: R3-R5 are all volatile so it is OK for this routine to destroy them
 *
 * INPUT: IrqMask (r3) - This set of DIE and 476 IRQ bits to enable
 * OUTPUT: void
 * Returns: void
 */
        .global iopinitEnableIRQ
iopinitEnableIRQ:
        /* Also disable PPC decrementer interrupt */
        lwi2    (r4,(PPC476_TCR_DIE))
        and.    r5,r3,r4
        beq     EnableOtherIrqs    /* DIE Bit not set, branch */
        mftcr   r5
        or      r5,r5,r4
        mttcr   r5                 /* Set DIE bit */
EnableOtherIrqs:
        /* Re-enable all the interrupts indicated by the IrqMask (r3) */
        mfmsr   r5
        lwi      (r4, IOC_PPC476_INTERRUPTS)
        and.    r3,r3,r4           /* Mask off bits not supported from IrqMask*/
        beq     DoneWithEnableIrqs /* Branch if no irq's to enable */
        or      r5, r5, r3
        mtmsr   r5
DoneWithEnableIrqs:
        blr
