/*******************************************************************************

NAME            iopCommon.c
VERSION         %version: 3 %
UPDATE DATE     %date_modified: Thu Oct 17 11:30:30 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopCommon.c
 *          Title:  Common IOP functions
 *     Programmer:  Nilesh Wilanker
 *     Modified  :  Naresh Madhusudana
 *  Creation Date:  Aug 9, 2011
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  08/09/11  NW    Initial version.
 *  08/23/11  NW    Added function iopiDelayUSec().
 *  10/05/11  NM    Porting to Invader
 *
 * Description
 * ------------
 * This file contains the implementation of common IOP functions needed by 
 * different libraries.
 *
 *-------------------------------------------------------------------------
 */

#include <iopCommon.h>

/* ENABLE_UBOOTLIB_PRINTS can be passed in by makefile to optionally enable
 * output. Do not #undef here or that will break. */
#ifdef ENABLE_UBOOTLIB_PRINTS
/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: printChar()
* SUMMARY: Prints the character on UART console.
* SCOPE: local.
*
* DESCRIPTION:
* This function sends the character to UART console.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Ch : Character to be printed.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
static void printChar(S8 Ch)
{
    volatile U8 status = 0;

    /* wait until txempty */
    while ( (status & LSR_THREMPTY) == 0 )
    {
        status = *((volatile U8 *)(COM1_ADDRESS + LSR_REG));
    }

    *((volatile U8 *)(COM1_ADDRESS + THR_REG)) = Ch;

    if ( '\n' == Ch )
    {
        printChar('\r');
    }
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: printStr()
* SUMMARY: Prints the string on UART console.
* SCOPE: local.
*
* DESCRIPTION:
* This function prints the string on UART console.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* PtrString : Pointer to the string to be printed.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
static void printStr(const char *PtrString)
{
    for (; *PtrString ; ++PtrString)
    {
        printChar (*PtrString);
    }
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: printInt()
* SUMMARY: Prints the integer on UART console.
* SCOPE: local.
*
* DESCRIPTION:
* This function prints the integer value on UART console.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* Num  : Integer value to be printed.
* Base : Base for number.
* Sg   : Signed number.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
static void printInt(S32 Num, S32 Base, S32 Sg)
{
    char          print_buf[PRINT_BUF_LEN];
    register char *str;
    register S32  neg      = 0;
    register U32  localNum = Num;

    if ( Num == 0 )
    {
        print_buf[0] = '0';
        print_buf[1] = '\0';
        printStr(print_buf);
        return ;
    }

    if ((Sg) && (10 == Base) && (Num < 0) )
    {
        neg      = 1;
        localNum = -Num;
    }

    str = print_buf + PRINT_BUF_LEN - 1;
    *str = '\0';

    while (localNum)
    {
        *--str    = "0123456789abcdef"[localNum % Base];
        localNum /= Base;
    }

    if ( neg )
    {
        *--str = '-';
    }
    printStr(str);
}
#endif /* ENABLE_UBOOTLIB_PRINTS */

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: print()
* SUMMARY: Prints the formatted string.
* SCOPE: local.
*
* DESCRIPTION:
* This function prints the formatted string with variable number of arguments.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr  : Pointer to formatted string.
* ArgCnt  : Number of variable arguments.
* PtrArgs : Pointer to variable arguments.
*
* RETURNS:
* void
*
* ERROR: If nothing is mentioned after a % example %abcd then abcd should be printed 
* or it can be returned. This is not a critical code.
*
* RESTRICTIONS: only %d, %x,%X,%u,%c,%s works. %08d does not work.
*
* NOTES:
*
----------------------------------------------------------------------------*/
static void print(S32 *FmtStr, U32 ArgCnt, U32 *PtrArgs)
{
#ifdef ENABLE_UBOOTLIB_PRINTS
    register PS8 format = (PS8)(*FmtStr);
    char         scr[2];
    U32          arg;
    U32          argNum = ArgCnt;
    U32		 done = 0;

    for (; *format != 0; ++format)
    {
        if ( *format == '%' )
        {
            ++format;
            if ( *format == '\0' )
            {
                break;
            }

            if ( *format == '%' )
            {
                printChar(*format);
                continue;
            }

            if ( argNum )
            {
                arg = PtrArgs[ArgCnt - argNum];
                argNum--;
	     
	     while(1)
	     {
		done = 1;
                switch( *format )
                {
                    case 's':
                    {
                        register char *s = ((char *)arg);
                        printStr(s ? s : "(null)");
                        break;
                    }

                    case 'd':
                    {
                        printInt(arg, 10, 1);
                        break;
                    }

                    case 'x':
                    case 'X':
                    {
                        printInt(arg, 16, 0);
                        break;
                    }

                    case 'u':
                    {
                        printInt(arg, 10, 0);
                        break;
                    }

                    case 'c':
                    {
                        scr[0] = arg;
                        scr[1] = '\0';
                        printStr(scr);
                        break;
                    }
		    case '\n':
		    {
			return;
		    }
		    default:
		    {
			done = 0;
			break;
		    }
                }
		if(done == 0)
		{
			++format;		
		}
		else
		{
			break;
		}
	     }
            }
        }
        else
        {
            printChar (*format);
        }
    }
#endif /* ENABLE_UBOOTLIB_PRINTS */
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopiDiagConsolePrint0()
* SUMMARY: Prints the string with no variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the string with no variable arguments.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr: Formatted string to be printed.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void iopiDiagConsolePrint0(char *FmtStr)
{
    register S32 *varg   = (S32 *)(&FmtStr);
    U32          argCnt  = 0;
    U32          args[4] = {0};

    print(varg, argCnt, args);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopiDiagConsolePrint1()
* SUMMARY: Prints the formatted string with single variable argument.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the formatted string with single variable argument.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void iopiDiagConsolePrint1(char *FmtStr, U32 Arg1)
{
    register S32 *varg   = (S32 *)(&FmtStr);
    U32          argCnt  = 1;
    U32          args[4] = {0};

    args[0] = Arg1;

    print(varg, argCnt, args);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopiDiagConsolePrint2()
* SUMMARY: Prints the formatted string with 2 variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the formatted string with 2 variable arguments.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First  variable argument.
* Arg2   : Second variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void iopiDiagConsolePrint2(char *FmtStr, U32 Arg1, U32 Arg2)
{
    register S32 *varg   = (S32 *)(&FmtStr);
    U32          argCnt  = 2;
    U32          args[4] = {0};

    args[0] = Arg1;
    args[1] = Arg2;

    print(varg, argCnt, args);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopiDiagConsolePrint3()
* SUMMARY: Prints the formatted string with 3 variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the formatted string with 3 variable arguments.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First  variable argument.
* Arg2   : Second variable argument.
* Arg3   : Third  variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void iopiDiagConsolePrint3(char *FmtStr, U32 Arg1, U32 Arg2, U32 Arg3)
{
    register S32 *varg   = (S32 *)(&FmtStr);
    U32          argCnt  = 3;
    U32          args[4] = {0};

    args[0] = Arg1;
    args[1] = Arg2;
    args[2] = Arg3;

    print(varg, argCnt, args);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopiDiagConsolePrint4()
* SUMMARY: Prints the formatted string with 4 variable arguments.
* SCOPE: global.
*
* DESCRIPTION:
* Prints the formatted string with 4 variable arguments.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* FmtStr : Formatted string to be printed.
* Arg1   : First  variable argument.
* Arg2   : Second variable argument.
* Arg3   : Third  variable argument.
* Arg4   : Fourth variable argument.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES:
*
----------------------------------------------------------------------------*/
void iopiDiagConsolePrint4(char *FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4)
{
    register S32 *varg   = (S32 *)(&FmtStr);
    U32          argCnt  = 4;
    U32          args[4] = {0};

    args[0] = Arg1;
    args[1] = Arg2;
    args[2] = Arg3;
    args[3] = Arg4;

    print(varg, argCnt, args);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: iopinitDelayUSec()
* SUMMARY: Generates the delay in microseconds
* SCOPE: global.
*
* DESCRIPTION:
* This function generates the specified amount of delay in microseconds.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* USec : Delay amount in microseconds.
*
* RETURNS:
* void
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the iop function called by DDR3 library.
*
----------------------------------------------------------------------------*/
void iopinitDelayUSec(U32 USec)
{
    U32 timeOut;
    if (USec == 0)
    {
        return;
    }

    USec += 1;
    timeOut = PPC476_ADD_USEC_TO_TICKS(USec);
    while (!PPC476_CHECK_TICKS(timeOut));
}
