/*******************************************************************************

NAME            ubootLib.c
VERSION         %version: 2 %
UPDATE DATE     %date_modified: Thu Oct 17 11:35:03 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  ubootLib.c
 *          Title:  Standalone u-boot library file
 *     Programmer:  Nilesh Wilanker
 *     Modified  :  Naresh Madhusudana
 *  Creation Date:  July 4, 2011
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  07/04/11  NW   Initial version.
 *  10/06/11  NM   Porting this file for Invader
 *  03/27/12  JM   LSISAS3108 Dual core library changes.
 *  05/31/12  JM   Added switch case DDR3_SIZE in function primUbootLibEntry.
 *  07/12/12  AM   Root Complex OSS changes.
 *
 * Description
 * ------------
 * This is the main file for u-boot standalone module. No functions should
 * be added to this file, else the jump location for the standalone module
 * might change.
 *
 *-------------------------------------------------------------------------
 */

#include <ddr3Init.h>     /* Memory controller initialization definitions */
#include <ubootLib.h>     /* U-Boot - Standalone Header                   */
#include <iopCommon.h>    /* Common IOP functions definitions             */
#include <primUbootLib.h> /* Primary U-Boot  - Standalone Header          */
#include <secUbootLib.h>  /* Secondary U-Boot  - Standalone Header        */
#include "rootComplex.h"  /* Root Complex API header                      */

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: primUbootLibEntry ()
* SUMMARY: Initializes the DDR3.
* SCOPE: global.
*
* DESCRIPTION:
* This function is the entry point function and calls the iop functions
* depending on the library name passed.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* argc: number of variable arguments
* argv: pointer to variable arguments
*
* RETURNS:
* 0            : Failure
* DC_FAIL      : primUbootLib : Dual Core function failed.
* DC_SUCCESS   : primUbootLib : Dual Core function success.
* INVALID_LIB  : primUbootLib : Invalid U-Boot library
* Size of DDR3 : on success
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the main function (entry point) for standalone
* module.
* argv[0]: Address of starting point of this standalone module
* argv[1]: Type of library.
* argv[2]: Function name
* argv[3]...argv[n-1]: Arguments needed by library functions.
----------------------------------------------------------------------------*/
U32 primUbootLibEntry (int argc, char *argv[])
{
    U32 result  = 0;
    U32 library = 0;
    U32 libAPI  = 0;

    if (argc < 2)
    {
        return 1;
    }

    library = *(int *)argv[1];
    libAPI  = *(int *)argv[2];

    if(library == DDR3_LIB) /* DDR3_LIB : 1 */
    {
        switch (libAPI)
        {
            case DDR3_INIT: /* DDR3_INIT : 1 */
                result = iopDDRInit (argc - 2, argv + 2);
                break;

            case DDR3_SIZE: /* DDR3_SIZE : 2 */
                result = iopDDRSize (argc - 2, argv + 2);
                break;

            default:
                iopiDiagConsolePrint0("Unknown Library API for DD3_LIB\n");
                result = 0;
                break;
        }
    }
    else if(library == MINI_DMA_LIB) /* MINI_DMA_LIB : 2 */
    {
        switch (libAPI)
        {
            case MINI_DMA_RESET_DDR: /* MINI_DMA_RESET_DDR : 1 */
                result = ddrPostUseDma(argc-2, argv+3);
                break;

            default:
                iopiDiagConsolePrint0("Unknown Library API for MINI_DMA_LIB\n");
                result = 0;
                break;
        }
    }
    else if( library == RC_LIB )
    {
        result = rootComplexWrapper(argc-2,argv+2);
    }
    /* For Primary U-Boot (Dual core operation) */
    else if(library == DC_PRIMARY_UBOOT_LIB) /* DC_PRIMARY_UBOOT_LIB : 3 */
    {
        result = primUbootLib(argc - 2, argv + 2);
    }
    else
    {
        iopiDiagConsolePrint0("Invalid Library for Primary U-Boot!\n");
        result = INVALID_LIB;
    }

    return (result);
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME: secUbootLibEntry()
* SUMMARY: Entry point for Secondary U-boot.
* SCOPE: global.
*
* DESCRIPTION:
* This function is the entry point function and invoke the Secondary U-boot
* APIs depending on the library name passed.
*
* Pre-conditions:
* N/A
*
* Post-conditions:
* N/A
*
* ARGUMENTS:
* argc : number of variable arguments
* argv : pointer to variable arguments
*
* RETURNS:
* DC_FAIL      : secUbootLib : Dual Core function failed.
* DC_SUCCESS   : secUbootLib : Dual Core function success.
* INVALID_LIB  : Invalid U-Boot library
*
* ERROR:
*
* RESTRICTIONS:
*
* NOTES: This is the main function (entry point) for Secondary U-boot
* module.
* argv[0]: Address of starting point of this Secondary U-boot module.
* argv[1]: Type of library.
* argv[2]: Function name.
* argv[3]...argv[n-1]: Arguments needed by library functions.
----------------------------------------------------------------------------*/
U32 secUbootLibEntry ( int argc, char *argv[] )
{
    U32 result;
    U32 library;

    library = *(int *)argv[1];

    /* For Secondary U-Boot (Dual core operation) */
    if(library == DC_SECONDARY_UBOOT_LIB)
    {
        result = secUbootLib(argc - 2, argv + 2);
    }
    else
    {
        iopiDiagConsolePrint0("Invalid Library for Secondary U-Boot!\n");
        result = INVALID_LIB;
    }

    return (result);
}
